import logging
from datetime import timedelta

from _add_lib_to_path import add_lib_to_path

# This function adds the project root to PATH
add_lib_to_path()
from sfm2.interface.sfm2_init_options import Sfm2CollectionInitOptions
from sfm2.utils.time_sync import TimeSync
from sfm2.sfm2_factory import Sfm2Factory

logging.basicConfig(level=logging.INFO)

options = Sfm2CollectionInitOptions(
    allow_duplicate_keys=False,
    key_by_id_not_name=True
)

all_sfm2s = Sfm2Factory.connect_all_found(collection_options=options)

if len(all_sfm2s) == 0:
    print("Could not connect to any SFM2")
    exit(-1)

sync = TimeSync()
status = sync.sync(all_sfm2s)  # Starts time synchronization

print(f"SFM2s clocks are now being synchronized to the PC time. Timestamp at 0: {sync.zero_timestamp}")

status.offset.subscribe(lambda o: print(f"Current maximum deviation: {o.total_seconds()}s"))

print("Waiting for accurate synchronization...")
print("You can speed up the synchronization by passing TimeSyncOptions when creating TimeSync")

# Waits until synchronization is within limits. (TimeSyncOptions.max_allowed_offset_ticks).
# You are not required to call this function.
status.wait_for_sync(timeout=timedelta(seconds=20))

print("Synchronized!")

input("Click enter twice to exit\n")

# This stops the time synchronization. You are not required to call this every time.
status.dispose()
