import logging

from _add_lib_to_path import add_lib_to_path

# This function adds the project root to PATH
add_lib_to_path()

from sfm2.interface.sfm2_init_options import Sfm2CollectionInitOptions
from sfm2.interface.config_types import SampleRate
from sfm2.sfm2_factory import Sfm2Factory
from sfm2.utils.throughput import calculate_throughput

logging.basicConfig(level=logging.WARNING)

options = Sfm2CollectionInitOptions(
    ignore_connection_errors=True,
    allow_duplicate_keys=True,
    key_by_id_not_name=False
    # When this option is set to True then the SFM2s in a Sfm2Collection are accessed by ID (Serial port name)
    # using the brackets operator.
    # When it's set to False (the default value) then they are accessed by the device name.
    # In real applications consider leaving setting this option to False (or not setting it at all).
    # Instead give each SFM a unique, meaningful name using the Sfm2.name property (or manually using the NAME= command).
)

all_sfm2s = Sfm2Factory.connect_all_found(collection_options=options)

if len(all_sfm2s) == 0:
    print("Could not connect to any SFM2")
    exit(-1)

for sfm2 in all_sfm2s:
    calculate_throughput((sfm2.acc.data, sfm2.gyr.data)) \
        .subscribe(lambda th, label=sfm2.id: print(f"{label} {th[0]} {th[1]}"))

for sr in SampleRate:
    if sr == SampleRate.OFF:
        continue
    print(f"Setting sample rate to {sr.value}Hz")

    for sfm2 in all_sfm2s:
        sfm2.acc.rate = sr
        sfm2.gyr.rate = sr

    if sr == SampleRate.MAX:
        input("Click enter twice to exit\n")
    else:
        input("Click enter twice to increase sample rate\n")

print("Resetting...")

for sfm2 in all_sfm2s:
    sfm2.reset()
