import logging

from _add_lib_to_path import add_lib_to_path

# This function adds the project root to PATH
add_lib_to_path()

from sfm2.interface.sfm2_init_options import Sfm2CollectionInitOptions
from sfm2.sfm2_factory import Sfm2Factory

logging.basicConfig(level=logging.INFO)

options = Sfm2CollectionInitOptions(
    ignore_connection_errors=True,
    allow_duplicate_keys=False,
    key_by_id_not_name=True
    # When this option is set to True then the SFM2s in a Sfm2Collection are accessed by ID (Serial port name)
    # using the brackets operator.
    # When it's set to False (the default value) then they are accessed by the device name.
    # In real applications consider leaving setting this option to False (or not setting it at all).
    # Instead give each SFM a unique, meaningful name using the Sfm2.name property (or manually using the NAME= command).
)

all_sfm2s = Sfm2Factory.connect_all_found(collection_options=options)

if len(all_sfm2s) == 0:
    print("Could not connect to any SFM2")
    exit(-1)

for sfm2 in all_sfm2s:
    print(f"Setting up {sfm2}")
    sfm2.sensor_fusion.rate = 13
    sfm2.sensor_fusion.quaternion.subscribe(lambda q, label=sfm2.id: print(f"{label}: {q}"))

# You can also access each SFM2 by its serial port (or device name if key_by_id_not_name=False):
# all_sfm2s["COM3"].sensor_fusion.rate = 26

input("Click enter twice to exit\n")
