import logging
from enum import Enum
from time import time, sleep

import sfm2._implementation.commands.command_list as cmds
from sfm2._implementation.commands.command_interface import CommandInterface
from sfm2.interface.exceptions import InvalidStateException
from sfm2.interface.sfm2_init_options import Sfm2InitOptions

_log = logging.getLogger(__name__)

_POLLING_INTERVAL_S = 1


class _State(Enum):
    NOT_RUN = "NOT_RUN"
    RUNNING = "RUNNING"
    PASSED = "PASSED"
    FAILED = "FAILED"


def run_self_test(cmd: CommandInterface, options: Sfm2InitOptions) -> bool:
    _log.info("Starting self-test")
    state_after_started = cmd.command(cmds.selftest)
    if _State(state_after_started) != _State.RUNNING:
        InvalidStateException("Self-test not running after being started").raise_and_log_error()

    start_time = time()
    while time() - start_time < options.self_test_timeout_s:
        sleep(_POLLING_INTERVAL_S)
        state_str = cmd.query(cmds.selftest)
        state = _State(state_str)
        if state == _State.PASSED:
            _log.info("Self-test passed")
            return True
        elif state == _State.FAILED:
            _log.warning("Self-test failed. Remember to keep the device still during self-test.")
            return False
        elif state != _State.RUNNING:
            InvalidStateException(
                f"Unexpected self-test state: {state}"
            ).raise_and_log_error()

    InvalidStateException(
        f"Self-test timeouted. Results not available in {options.self_test_timeout_s}"
    ).raise_and_log_error()
