import logging
from typing import Union

from rx.core.typing import Observable

from sfm2._implementation.internal_utils.find_sample_rate import find_sample_rate
from sfm2.interface.sample_types import VectorSample
from sfm2.interface.sfm2_config import SampleRate, Sfm2Config
from sfm2.interface.sfm2_data import Sfm2Data
from sfm2.interface.sfm2_magnetometer import Sfm2Magnetometer

_log = logging.getLogger(__name__)


class Sfm2MagnetometerImpl(Sfm2Magnetometer):
    def __init__(self, config: Sfm2Config, data: Sfm2Data):
        self._config = config
        self._data = data

    @property
    def data(self) -> Observable[VectorSample]:
        return self._data.md

    @property
    def rate(self) -> SampleRate:
        return self._config.msr

    @rate.setter
    def rate(self, rate: Union[SampleRate, float]) -> None:
        found_rate = find_sample_rate(rate, maximum=SampleRate.HZ_104)
        self._configure_acc_if_necessary(found_rate)
        self._config.msr = rate

    def _configure_acc_if_necessary(self, requested_msr: SampleRate) -> None:
        can_set_now = self._can_set_msr(requested_msr)
        if not can_set_now:
            _log.info("Can't set requested magnetometer sample rate, configuring accelerometer first")
            self._config.asr = requested_msr

    def _can_set_msr(self, requested_msr: SampleRate) -> bool:
        return self._config.asr >= requested_msr \
               or self._config.gsr >= requested_msr
