import logging
from typing import Optional

import sfm2._implementation.commands.command_list as cmds
from sfm2._implementation.commands.command_interface import CommandInterface
from sfm2.interface.exceptions import InvalidArgumentException
from sfm2.interface.sfm2_init_options import Sfm2InitOptions

_log = logging.getLogger(__name__)

_MAX_LENGTH = 16


class Sfm2Name:
    def __init__(self, cmd: CommandInterface, options: Sfm2InitOptions):
        self._cmd = cmd
        self._cached: Optional[str] = None
        self._cache_name = options.cache_name

    def get(self) -> str:
        if self._cached is None or not self._cache_name:
            self._cached = self._read()
            _log.debug(f"Name read: {self._cached}")

        return self._cached

    def set(self, value: str) -> None:
        _log.debug(f"Setting name to {value}")
        self._raise_if_invalid(value)
        self._cmd.set_value_exact(cmds.name, value)
        self._cached = value

    def _read(self) -> str:
        return self._cmd.query(cmds.name, check_if_failed=False)

    @staticmethod
    def _raise_if_invalid(value: str) -> None:
        printable_ascii = value.isascii() and value.isprintable()
        if not printable_ascii:
            InvalidArgumentException("All characters must be printable ASCII") \
                .raise_and_log_warning(log=_log)

        if len(value) > _MAX_LENGTH:
            InvalidArgumentException(f"The name must not be longer than {_MAX_LENGTH}") \
                .raise_and_log_warning(log=_log)
