import logging

import sfm2._implementation.commands.command_list as cmds
from sfm2._implementation.commands.command_interface import CommandInterface
from sfm2.interface.config_types import SampleRate
from sfm2.interface.exceptions import InvalidStateException
from sfm2.interface.sfm2_config import Sfm2Config
from sfm2.interface.sfm2_reference import Sfm2Reference

_log = logging.getLogger(__name__)


class Sfm2ReferenceImpl(Sfm2Reference):
    def __init__(self, cmd: CommandInterface, config: Sfm2Config):
        self._cmd = cmd
        self._config = config

    def tare_now(self) -> None:
        _log.debug("Taring")
        self._raise_if_sf_disabled()
        self._cmd.command(cmds.tare)

    def headtare_now(self) -> None:
        _log.debug("Headtaring")
        self._raise_if_sf_disabled()
        self._cmd.command(cmds.headtare)

    @property
    def headtare(self) -> float:
        value_str = self._cmd.query(cmds.headtare)
        return float(value_str)

    @headtare.setter
    def headtare(self, value: float) -> None:
        _log.debug(f"Setting headtare: {value}")
        self._cmd.set_value(cmds.headtare, str(value))

    @property
    def globref(self) -> bool:
        value_str = self._cmd.query(cmds.globref)
        return bool(value_str)

    @globref.setter
    def globref(self, enabled: bool) -> None:
        _log.debug(f"Setting globref: {enabled}")
        value_str = "1" if enabled else "0"
        self._cmd.set_value_exact(cmds.globref, value_str)

    def _raise_if_sf_disabled(self) -> None:
        if self._config.sfor == SampleRate.OFF:
            InvalidStateException("Sensor Fusion is not enabled") \
                .raise_and_log_warning(log=_log)
