import logging
from typing import Tuple

import sfm2._implementation.commands.command_list as cmds
from sfm2._implementation.commands.command_interface import CommandInterface
from sfm2.interface.exceptions import UnsupportedFirmwareVersionException

_log = logging.getLogger(__name__)


def check_firmware_version(cmd: CommandInterface, api_version_range: Tuple[int, int], skip: bool) -> None:
    if skip:
        _log.warning("Skipping firmware version check.")
        return

    device_firmware_version = cmd.query(cmds.fwver)
    _log.info(cmds.fwver.set_value_str(device_firmware_version))
    if device_firmware_version.startswith("0"):
        UnsupportedFirmwareVersionException("Firmware versions before 1.0 are not supported."
                                            " Update the device firmware.") \
            .raise_and_log_error(_log)
    device_api_version = cmd.query(cmds.apiver)
    _log.info(cmds.apiver.set_value_str(device_api_version))
    if int(device_api_version) < api_version_range[0]:
        UnsupportedFirmwareVersionException(
            f"Device API version too old (actual {device_api_version}, minimum supported {api_version_range[0]})."
            " Update the device firmware.").raise_and_log_error(_log)
    if int(device_api_version) > api_version_range[1]:
        UnsupportedFirmwareVersionException(
            f"Device API version too new (actual {device_api_version}, maximum supported {api_version_range[1]})."
            " Update the Python library.").raise_and_log_error(_log)

    _log.debug("Firmware check passed")
