import logging
from queue import Queue
from threading import Thread
from typing import Callable

from rx.core.abc import Disposable

from sfm2.interface.exceptions import InvalidArgumentException

_log = logging.getLogger(__name__)


class BackgroundThreadExecutor(Disposable):
    def __init__(self):
        self._queue = Queue()

        thread = Thread(target=self._run)
        thread.daemon = True
        thread.start()

    def execute(self, action: Callable[[], None]) -> None:
        if action is None:
            InvalidArgumentException("Action can't be none").raise_and_log_error()

        self._queue.put_nowait(action)

    def dispose(self):
        self._queue.put_nowait(None)

    def _run(self) -> None:
        _log.debug("Starting execution thread")

        while True:
            action = self._queue.get()
            if action is None:
                break

            try:
                action()
            except Exception as _:
                _log.exception(f"Executing action {action} failed")

        _log.debug("Execution thread completed")
