import logging

from rx.core.abc import Disposable

_log = logging.getLogger(__name__)


class DisposeOnException:
    def __init__(self):
        self._disposables = list()

    def add(self, d: Disposable) -> None:
        self._disposables.append(d)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_inst, ex_tb) -> bool:
        if exc_inst is not None:
            _log.info(f"Exception encountered. Disposing {len(self._disposables)} items.")
            self._dispose_all()
        self._disposables.clear()
        return False

    def _dispose_all(self) -> None:
        for d in self._disposables:
            try:
                d.dispose()
            except Exception as e:
                _log.exception(f"Could not dispose {d}")
