import logging
import math
from typing import Union, Optional, Tuple

from sfm2.interface.sfm2_config import SampleRate

_log = logging.getLogger(__name__)


def find_sample_rate(rate: Union[SampleRate, float], maximum: SampleRate = SampleRate.MAX) -> SampleRate:
    if isinstance(rate, SampleRate):
        return rate

    found = _find_for_float(rate, maximum)
    _log.info(f"For requested numeric sample rate: {rate} found available: {found}")
    return found


def _find_for_float(rate: float, maximum: SampleRate) -> SampleRate:
    if rate is None or math.isnan(rate):
        return SampleRate.OFF

    lower, higher = _find_lower_and_higher_rate(rate, maximum)

    if higher is None:
        return maximum
    elif higher == SampleRate.HZ_0:
        return SampleRate.HZ_0
    elif lower == SampleRate.HZ_0 and rate > 0:
        return higher

    diff_to_higher = higher.value - rate
    diff_to_lower = rate - lower.value / 2

    return higher if diff_to_higher <= diff_to_lower else lower


def _find_lower_and_higher_rate(value: float, maximum: SampleRate) -> Tuple[SampleRate, Optional[SampleRate]]:
    previous = SampleRate.HZ_0
    for rate in SampleRate:
        if rate.value > maximum.value:
            break
        if rate.value >= value:
            return previous, rate

        previous = rate

    return previous, None
