from threading import Thread
from typing import Callable

import rx
from rx.core.abc import Observable, Disposable, Observer

from sfm2._implementation.internal_utils.cancellation_token import CancellationToken
from sfm2._implementation.internal_utils.thread_raw_subject import ThreadRawSubject


def create_observable_on_new_thread(blocking_subscribe: Callable[[Observer, CancellationToken], None]) -> Observable:
    def wrap_blocking_subscribe(observer: Observer, ct: CancellationToken) -> None:
        try:
            blocking_subscribe(observer, ct)
            observer.on_completed()
        except Exception as e:
            observer.on_error(e)

    def subscribe(observer: Observer) -> Disposable:
        ct = CancellationToken()
        th = Thread(target=lambda: wrap_blocking_subscribe(observer, ct))
        th.daemon = True
        th.start()

        return ct

    return rx.create(subscribe)


def observe_on_new_thread(observable: Observable) -> Observable:
    return rx.using(lambda: _HotThreadSubject(observable), lambda s: s.get_subject())


class _HotThreadSubject(Disposable):
    def __init__(self, observable: Observable):
        self._subject = ThreadRawSubject()
        self._subscription = observable.subscribe(self._subject)

    def get_subject(self) -> ThreadRawSubject:
        return self._subject

    def dispose(self):
        self._subscription.dispose()
        self._subject.dispose()
