import functools
import logging
from typing import Type, Optional, Callable, Any

from sfm2.interface.exceptions import Sfm2Exception, Sfm2InternalException

_log = logging.getLogger(__name__)


def wrap_exception(exc_to_catch: Type = Exception, wrapping_exc: Optional[Type] = None,
                   create_wrapping_exc: Optional[Callable[[Any], Exception]] = None, log=_log):
    none_defined = wrapping_exc is None and create_wrapping_exc is None
    both_defined = wrapping_exc is not None and create_wrapping_exc is not None
    wrong_type = wrapping_exc is not None and not issubclass(wrapping_exc, Sfm2Exception)
    if none_defined or both_defined or wrong_type:
        Sfm2InternalException("Invalid arguments").raise_and_log_error(log=log)

    def create(*args, **kwargs) -> Sfm2Exception:
        if create_wrapping_exc is not None:
            exc = create_wrapping_exc(*args, **kwargs)
            if not isinstance(exc, Sfm2Exception):
                return Sfm2InternalException(f"Wrapping exception must be {Sfm2Exception.__name__}")
            return exc

        return wrapping_exc()

    def decorator(function):
        @functools.wraps(function)
        def wrapper(*args, **kwargs):
            try:
                return function(*args, **kwargs)
            except exc_to_catch as e:
                create(*args, **kwargs).raise_and_log_error(cause=e, log=log)

        return wrapper

    return decorator
