class BytesStream:
    def __init__(self):
        self._content = bytes()
        self._examined = 0

    def append(self, data: bytes) -> None:
        self._content += data

    def unexamined(self) -> int:
        return len(self.content) - self._examined

    def consume_examined(self) -> bytes:
        return self.consume(self._examined)

    def consume(self, size: int) -> bytes:
        consumed = self._content[:size]
        self._content = self._content[size:]
        self._examined -= size
        if self._examined < 0:
            self._examined = 0
        return consumed

    def any(self) -> bool:
        return len(self._content) != 0

    def __len__(self):
        return len(self._content)

    @property
    def content(self):
        return self._content

    def __iter__(self):
        return self

    def __next__(self) -> int:
        if self._examined < len(self._content):
            el = self._content[self._examined]
            self._examined += 1
            return el
        else:
            raise StopIteration

    def all_examined(self) -> None:
        self._examined = len(self._content)
