from __future__ import annotations

from dataclasses import dataclass
from datetime import timedelta
from enum import Enum


class PressureSampleRate(Enum):
    OFF = 0
    HZ_0 = 0
    HZ_1 = 1
    HZ_10 = 10
    HZ_25 = 25
    HZ_50 = 50
    HZ_75 = 75
    HZ_100 = 100
    MAX = 100


class TemperatureSamplingMode(Enum):
    OFF = "OFF"
    INTERVAL = "INTERVAL"
    CONTINUOUS = "CONTINUOUS"


@dataclass(frozen=True)
class TemperatureSamplingSetup:
    """
    Holds the complete setup of temperature/humidity sensor. Use one of the static factory methods to easily create an instance.
    """

    @staticmethod
    def off() -> TemperatureSamplingSetup:
        return TemperatureSamplingSetup(TemperatureSamplingMode.OFF, timedelta())

    @staticmethod
    def interval(interval: timedelta) -> TemperatureSamplingSetup:
        return TemperatureSamplingSetup(TemperatureSamplingMode.INTERVAL, interval)

    @staticmethod
    def interval_s(seconds: float) -> TemperatureSamplingSetup:
        return TemperatureSamplingSetup.interval(timedelta(seconds=seconds))

    @staticmethod
    def continuous() -> TemperatureSamplingSetup:
        return TemperatureSamplingSetup(TemperatureSamplingMode.CONTINUOUS, timedelta())

    mode: TemperatureSamplingMode
    sampling_interval: timedelta
