from abc import ABC, abstractmethod
from datetime import timedelta
from typing import Union

from sfm2.interface.config_types import SampleRate, AccelFullRange, GyroFullRange
from sfm2.interface.env_sensor_config_types import PressureSampleRate, TemperatureSamplingMode


class Sfm2Config(ABC):
    """
    Allows to access the SFM2 config.
    Each property corresponds to and is named after a command.
    """

    @property
    @abstractmethod
    def asr(self) -> SampleRate:
        pass

    @asr.setter
    @abstractmethod
    def asr(self, rate: Union[SampleRate, float]) -> None:
        pass

    @property
    @abstractmethod
    def afr(self) -> AccelFullRange:
        pass

    @afr.setter
    @abstractmethod
    def afr(self, value: AccelFullRange) -> None:
        pass

    @property
    @abstractmethod
    def gsr(self) -> SampleRate:
        pass

    @gsr.setter
    @abstractmethod
    def gsr(self, rate: Union[SampleRate, float]) -> None:
        pass

    @property
    @abstractmethod
    def gfr(self) -> GyroFullRange:
        pass

    @gfr.setter
    @abstractmethod
    def gfr(self, value: GyroFullRange) -> None:
        pass

    @property
    @abstractmethod
    def msr(self) -> SampleRate:
        pass

    @msr.setter
    @abstractmethod
    def msr(self, rate: Union[SampleRate, float]) -> None:
        pass

    @property
    @abstractmethod
    def sfor(self) -> SampleRate:
        pass

    @sfor.setter
    @abstractmethod
    def sfor(self, rate: Union[SampleRate, float]) -> None:
        pass

    @property
    @abstractmethod
    def sfop(self) -> int:
        pass

    @sfop.setter
    @abstractmethod
    def sfop(self, prescaler: int) -> None:
        pass

    @property
    @abstractmethod
    def psr(self) -> PressureSampleRate:
        pass

    @psr.setter
    @abstractmethod
    def psr(self, rate: Union[PressureSampleRate, int]) -> None:
        pass

    @property
    @abstractmethod
    def altzero(self) -> float:
        pass

    @altzero.setter
    @abstractmethod
    def altzero(self, zero_altitude_pressure_hpa: float) -> None:
        pass

    @property
    @abstractmethod
    def pln(self) -> bool:
        pass

    @pln.setter
    @abstractmethod
    def pln(self, low_noise_enabled: bool) -> None:
        pass

    @property
    @abstractmethod
    def tint(self) -> timedelta:
        pass

    @tint.setter
    @abstractmethod
    def tint(self, sampling_interval: timedelta) -> None:
        pass

    @property
    @abstractmethod
    def tmode(self) -> TemperatureSamplingMode:
        pass

    @tmode.setter
    @abstractmethod
    def tmode(self, mode: TemperatureSamplingMode) -> None:
        pass
