from abc import ABC, abstractmethod
from typing import Union

from rx.core.typing import Observable

from sfm2.interface.sample_types import VectorSample
from sfm2.interface.sfm2_config import SampleRate, GyroFullRange


class Sfm2Gyroscope(ABC):
    """
    High-level interface for SFM2 gyroscope
    """

    @property
    @abstractmethod
    def data(self) -> Observable[VectorSample]:
        """
        Gets the gyroscope output (GD)

        :rtype: Observable[VectorSample]
        """
        pass

    @property
    @abstractmethod
    def rate(self) -> SampleRate:
        """
        Reads or sets gyroscope sampling rate.

        Can be set to a float value, in this case the closest available setting will be applied.

        :type: Union[SampleRate, float]
        :rtype: SampleRate
        """
        pass

    @rate.setter
    @abstractmethod
    def rate(self, rate: Union[SampleRate, float]) -> None:
        pass

    @property
    @abstractmethod
    def range(self) -> GyroFullRange:
        """
        Reads or sets gyroscope range. See :class:`~sfm2.interface.config_types.GyroFullRange`

        :type: AccelFullRange
        """
        pass

    @range.setter
    @abstractmethod
    def range(self, value: GyroFullRange) -> None:
        pass
