from abc import ABC, abstractmethod
from typing import Union

from rx.core.typing import Observable

from sfm2.interface.sample_types import VectorSample
from sfm2.interface.sfm2_config import SampleRate


class Sfm2Magnetometer(ABC):
    """
    High-level interface for SFM2 magnetometer
    """

    @property
    @abstractmethod
    def data(self) -> Observable[VectorSample]:
        """
        Gets the magnetometer output (MD)

        :rtype: Observable[VectorSample]
        """
        pass

    @property
    @abstractmethod
    def rate(self) -> SampleRate:
        """
        Reads or sets magnetometer sampling rate.

        Can be set to a float value, in this case the closest available setting will be applied.
        Setting this rate will automatically configure accelerometer sample rate if necessary.

        :type: Union[SampleRate, float]
        :rtype: SampleRate
        """
        pass

    @rate.setter
    @abstractmethod
    def rate(self, rate: Union[SampleRate, float]) -> None:
        pass
