from abc import abstractmethod, ABC


class Sfm2Reference(ABC):
    """
    Can be used to configure reference frame
    """

    @abstractmethod
    def tare_now(self) -> None:
        """
        Tare the sensor in current position.
        Sensor fusion must be running, you should also make sure it converged before calling this function.

        :raises InvalidStateException: If sensor fusion is not running.
        """
        pass

    @abstractmethod
    def headtare_now(self) -> None:
        """
        Tare the sensor heading in current position.
        Sensor fusion must be running, you should also make sure it converged before calling this function.

        :raises InvalidStateException: If sensor fusion is not running.
        """
        pass

    @property
    @abstractmethod
    def headtare(self) -> float:
        """
        Gets or sets the sensor heading tare.

        Heading tare is given in degrees.

        :type: float
        """
        pass

    @headtare.setter
    @abstractmethod
    def headtare(self, value: float) -> None:
        pass

    @property
    @abstractmethod
    def globref(self) -> bool:
        """
        Enables global reference (GLOBREF) mode.

        When enabled, all vector outputs are given in global reference frame.
        When enabled, all vector outputs are given in SFM2s local reference frame.

        GLOBREF does not affect rotational outputs.

        :type: bool
        """
        pass

    @globref.setter
    @abstractmethod
    def globref(self, enabled: bool) -> None:
        pass
