from abc import abstractmethod, ABC
from datetime import timedelta
from typing import Union

from rx.core.typing import Observable


class Sfm2Time(ABC):
    """
    Allows to read and manipulate the SFM2 time.
    """

    @abstractmethod
    def get(self) -> timedelta:
        """
        Reads the time.

        :rtype: timedelta
        """
        pass

    @abstractmethod
    def set(self, ts: timedelta) -> None:
        """
        Sets the time.


        :param ts: Time to be set
        :type ts: timedelta
        """
        pass

    @abstractmethod
    def reset(self) -> None:
        """
        Sets time to 0.

        """
        pass

    @abstractmethod
    def apply_offset(self, offset: Union[timedelta, int]) -> None:
        """
        Offsets time by a given time delta.

        :param offset: Time offset to be applied
        :type offset: timedelta
        """
        pass

    @property
    @abstractmethod
    def reports(self) -> Observable[timedelta]:
        pass

    @property
    @abstractmethod
    def reports_rate_hz(self) -> float:
        pass

    @reports_rate_hz.setter
    @abstractmethod
    def reports_rate_hz(self, value: float) -> None:
        pass

    @property
    @abstractmethod
    def trim(self) -> int:
        pass

    @trim.setter
    @abstractmethod
    def trim(self, value: int) -> None:
        pass

    @abstractmethod
    def calibrate(self, time_passed_on_sfm2: timedelta, time_passed_on_pc: timedelta) -> None:
        pass
